@@ -4,11 +4,11 @@
 
 ;; Author: Alvaro Ramirez https://xenodium.com
 ;; URL: https://github.com/xenodium/acp.el
-;; Package-Version: 20251116.1155
-;; Package-Revision: 3c6a39c09684
+;; Package-Version: 20251219.2135
+;; Package-Revision: 7b67facc657a
 ;; Package-Requires: ((emacs "28.1"))
 
-(defconst acp-package-version "0.7.2")
+(defconst acp-package-version "0.8.2")
 
 ;; This package is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -118,10 +118,16 @@ functions for advanced customization or testing."
                        :buffer stderr-buffer
                        :filter (lambda (_process raw-output)
                                  (acp--log client "STDERR" "%s" (string-trim raw-output))
-                                 (when-let ((api-error (acp--parse-stderr-api-error raw-output)))
+                                 (when-let ((std-error (cond
+                                                        ((acp--parse-stderr-api-error raw-output)
+                                                         (acp--parse-stderr-api-error raw-output))
+                                                        ((not (string-empty-p (string-trim raw-output)))
+                                                         ;; Fallback: create a generic error response
+                                                         `((code . -32603)
+                                                           (message . ,raw-output))))))
                                    (acp--log client "API-ERROR" "%s" (string-trim raw-output))
                                    (dolist (handler (map-elt client :error-handlers))
-                                     (funcall handler api-error)))))))
+                                     (funcall handler std-error)))))))
     (let ((process (make-process
                     :name (format "acp-client(%s)-%s"
                                   (map-elt client :command)
@@ -254,15 +260,16 @@ Note: These are agent process errors.
   "Shutdown ACP CLIENT and release resources."
   (unless client
     (error ":client is required"))
-  (unless (and (or (not (map-elt client :process))
-                   (process-live-p (map-elt client :process)))
-               (buffer-live-p (acp-logs-buffer :client client))
-               (buffer-live-p (acp-traffic-buffer :client client)))
-    (error "Client already shut down"))
-  (when (process-live-p (map-elt client :process))
-    (delete-process (map-elt client :process)))
-  (kill-buffer (acp-logs-buffer :client client))
-  (kill-buffer (acp-traffic-buffer :client client)))
+  (if (and (or (not (map-elt client :process))
+               (process-live-p (map-elt client :process)))
+           (buffer-live-p (acp-logs-buffer :client client))
+           (buffer-live-p (acp-traffic-buffer :client client)))
+      (progn
+        (when (process-live-p (map-elt client :process))
+          (delete-process (map-elt client :process)))
+        (kill-buffer (acp-logs-buffer :client client))
+        (kill-buffer (acp-traffic-buffer :client client)))
+    (message "Client already shut down")))
 
 (cl-defun acp-send-request (&key client request buffer on-success on-failure sync)
   "Send REQUEST from CLIENT.
@@ -437,16 +444,18 @@ and https://agentclientprotocol.com/protocol/schema#initializeresponse."
                                                                     t
                                                                   :false))))))))))
 
-(cl-defun acp-make-authenticate-request (&key method-id)
+(cl-defun acp-make-authenticate-request (&key method-id method)
   "Instantiate an \"authenticate\" request.
 
-METHOD-ID is the authentication method to use.
+METHOD-ID and METHOD (optionally) for authentication method to use.
 
 See https://agentclientprotocol.com/protocol/schema#authenticaterequest."
   (unless method-id
     (error ":method-id is required"))
   `((:method . "authenticate")
-    (:params . ((methodId . ,method-id)))))
+    (:params . ,(append `((methodId . ,method-id))
+                        (when method
+                          `((authMethod . ,method)))))))
 
 (cl-defun acp-make-session-new-request (&key cwd mcp-servers)
   "Instantiate a \"session/new\" request.
@@ -494,6 +503,23 @@ See https://agentclientprotocol.com/protocol/session-modes"
     (:params . ((sessionId . ,session-id)
                 (modeId . ,mode-id)))))
 
+(cl-defun acp-make-session-set-model-request (&key session-id model-id)
+  "Instantiate a \"session/set_model\" request.
+
+SESSION-ID is the ID of the session to change the model for.
+MODEL-ID is the model to set (e.g., \"default\", \"haiku\").
+
+This is a claude-code-acp extension to the ACP protocol.
+
+See https://docs.claude.com/en/api/agent-sdk/typescript"
+  (unless session-id
+    (error ":session-id is required"))
+  (unless model-id
+    (error ":model-id is required"))
+  `((:method . "session/set_model")
+    (:params . ((sessionId . ,session-id)
+                (modelId . ,model-id)))))
+
 (cl-defun acp-make-session-request-permission-response (&key request-id option-id cancelled)
   "Instantiate a \"session/request_permission\" response.
 
